@@ -301,8 +301,6 @@ PYBIND11_MODULE(py_sirius, m)
             .def(py::init<Simulation_context&>(), py::keep_alive<1, 2>(), "ctx"_a)
             .def("generate", &Potential::generate, "density"_a, "use_sym"_a, "transform_to_rg"_a)
             .def("fft_transform", &Potential::fft_transform)
-            .def("save", &Potential::save)
-            .def("load", &Potential::load)
             .def_property("vxc", py::overload_cast<>(&Potential::xc_potential),
                           py::overload_cast<>(&Potential::xc_potential), py::return_value_policy::reference_internal)
             .def_property("exc", py::overload_cast<>(&Potential::xc_energy_density),
@@ -468,9 +466,8 @@ PYBIND11_MODULE(py_sirius, m)
 
     py::class_<fft::Grid>(m, "FFT3D_grid")
             .def_property_readonly("num_points", py::overload_cast<>(&fft::Grid::num_points, py::const_))
-            .def_property_readonly("shape", [](const fft::Grid& obj) -> std::array<int, 3> {
-                return {obj[0], obj[1], obj[2]};
-            });
+            .def_property_readonly("shape",
+                                   [](const fft::Grid& obj) -> std::array<int, 3> { return {obj[0], obj[1], obj[2]}; });
 
     py::class_<mdarray<complex_double, 1>>(m, "mdarray1c")
             .def("on_device", &mdarray<complex_double, 1>::on_device)
