@@ -253,7 +253,8 @@ class Simulation_context : public Simulation_parameters
     memory_t host_memory_t_{memory_t::none};
 
     /// SPLA library context.
-    std::shared_ptr<::spla::Context> spla_ctx_{new ::spla::Context{SPLA_PU_HOST}};
+    /** Context is mutable to allow SPLA library to change it. */
+    mutable std::shared_ptr<::spla::Context> spla_ctx_{new ::spla::Context{SPLA_PU_HOST}};
 
     std::ostream* output_stream_{nullptr};
     std::ofstream output_file_stream_;
@@ -318,6 +319,8 @@ class Simulation_context : public Simulation_parameters
         init_common();
     }
 
+    /// Create an empty simulation context with an explicit communicators for k-point and
+    /// band parallelisation.
     Simulation_context(mpi::Communicator const& comm__, mpi::Communicator const& comm_k__,
                        mpi::Communicator const& comm_band__)
         : comm_(comm__)
@@ -327,29 +330,41 @@ class Simulation_context : public Simulation_parameters
         init_common();
     }
 
-    /// Create a simulation context with world communicator and load parameters from JSON string or JSON file.
-    Simulation_context(std::string const& str__)
-        : comm_(mpi::Communicator::world())
-    {
-        init_common();
-        import(str__);
-        unit_cell_->import(cfg().unit_cell());
-    }
-
-    explicit Simulation_context(nlohmann::json const& dict__)
-        : comm_(mpi::Communicator::world())
-    {
-        init_common();
-        import(dict__);
-        unit_cell_->import(cfg().unit_cell());
-    }
-
-    // /// Create a simulation context with world communicator and load parameters from JSON string or JSON file.
-    Simulation_context(std::string const& str__, mpi::Communicator const& comm__)
+    /// Create a simulation context with world communicator and load parameters from JSON string or a file.
+    explicit Simulation_context(std::string const& str__, mpi::Communicator const& comm__ = mpi::Communicator::world())
         : comm_(comm__)
     {
         init_common();
-        import(str__);
+        if (!is_json_string(str__) && isHDF5(str__)) {
+            HDF5_tree fin(str__, hdf5_access_t::read_only);
+            std::string json_string;
+            fin.read("config", json_string);
+            auto dict = read_json_from_file_or_string(json_string);
+            for (auto& e : dict["unit_cell"]["atom_types"]) {
+                auto label                             = e.get<std::string>();
+                dict["unit_cell"]["atom_files"][label] = "";
+            }
+            import(dict);
+            unit_cell_->import(cfg().unit_cell());
+
+            /* need to set type of calculation before parsing species */
+            this->electronic_structure_method(cfg().parameters().electronic_structure_method());
+            for (int iat = 0; iat < unit_cell_->num_atom_types(); iat++) {
+                fin["unit_cell"]["atom_types"][iat].read("config", json_string);
+                unit_cell_->atom_type(iat).read_input(json_string);
+            }
+        } else {
+            import(str__);
+            unit_cell_->import(cfg().unit_cell());
+        }
+    }
+
+    explicit Simulation_context(nlohmann::json const& dict__,
+                                mpi::Communicator const& comm__ = mpi::Communicator::world())
+        : comm_(comm__)
+    {
+        init_common();
+        import(dict__);
         unit_cell_->import(cfg().unit_cell());
     }
 
@@ -692,14 +707,8 @@ class Simulation_context : public Simulation_parameters
         return fft_coarse_grid_;
     }
 
-    auto const&
-    spla_context() const
-    {
-        return *spla_ctx_;
-    }
-
     auto&
-    spla_context()
+    spla_context() const
     {
         return *spla_ctx_;
     }
@@ -764,7 +773,7 @@ class Simulation_context : public Simulation_parameters
 
     /// Export parameters of simulation context as a JSON dictionary.
     nlohmann::json
-    serialize()
+    serialize() const
     {
         nlohmann::json dict;
         dict["config"] = cfg().dict();
