@@ -141,19 +141,19 @@ get_section_options(std::string const& section__)
     return input_schema["properties"][section__]["properties"];
 }
 
-void
-Simulation_parameters::import(std::string const& str__)
-{
-    auto json = read_json_from_file_or_string(str__);
-    import(json);
-}
-
 void
 Simulation_parameters::import(nlohmann::json const& dict__)
 {
     cfg_.import(dict__);
 }
 
+void
+Simulation_parameters::import(std::string const& str__)
+{
+    auto dict = read_json_from_file_or_string(str__);
+    this->import(dict);
+}
+
 void
 Simulation_parameters::import(cmd_args const& args__)
 {
@@ -174,6 +174,9 @@ Simulation_parameters::import(cmd_args const& args__)
 
     cfg_.iterative_solver().early_restart(
             args__.value("iterative_solver.early_restart", cfg_.iterative_solver().early_restart()));
+    cfg_.iterative_solver().energy_tolerance(
+            args__.value("iterative_solver.energy_tolerance", cfg_.iterative_solver().energy_tolerance()));
+    cfg_.iterative_solver().num_steps(args__.value("iterative_solver.num_steps", cfg_.iterative_solver().num_steps()));
     cfg_.mixer().beta(args__.value("mixer.beta", cfg_.mixer().beta()));
     cfg_.mixer().type(args__.value("mixer.type", cfg_.mixer().type()));
 }
