@@ -25,6 +25,7 @@ DFT_ground_state::initial_state()
     PROFILE("sirius::DFT_ground_state::initial_state");
 
     density_.initial_density();
+    density_.print_info(ctx_.out(1));
     potential_.generate(density_, ctx_.use_symmetry(), true);
     if (!ctx_.full_potential()) {
         if (ctx_.cfg().parameters().precision_wf() == "fp32") {
@@ -86,7 +87,7 @@ DFT_ground_state::energy_kin_sum_pw() const
             }
             ekin += 0.5 * d * kp->weight() * Gk.length2();
         } // igloc
-    }     // ikloc
+    } // ikloc
     ctx_.comm().allreduce(&ekin, 1);
     return ekin;
 }
@@ -238,6 +239,9 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
                                                          ctx_.cfg().iterative_solver().num_steps());
             /* find band occupancies */
             kset_.find_band_occupancies<double>();
+
+            auto vs = potential_.get_spherical_potential();
+            density_.generate_core_charge_density(vs);
             /* generate new density from the occupied wave-functions */
             density_.generate<double>(kset_, ctx_.use_symmetry(), true, true);
         }
@@ -348,7 +352,8 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         ctx_.message(2, __func__, out);
         /* check if the calculation has converged */
         bool converged{true};
-        converged = (std::abs(eold - etot) < energy_tol__) && result.converged && iter_solver_converged;
+        // converged = (std::abs(eold - etot) < energy_tol__) && result.converged && iter_solver_converged;
+        converged = (std::abs(eold - etot) < energy_tol__) && iter_solver_converged;
         if (ctx_.cfg().mixer().use_hartree()) {
             converged = converged && (eha_res < density_tol__);
         } else {
@@ -379,7 +384,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
                 density_.mag(j).rg().fft_transform(-1);
             }
         }
-        potential_.save(storage_file_name);
+        // potential_.save(storage_file_name);
         density_.save(storage_file_name);
         // kset_.save(storage_file_name);
     }
@@ -430,7 +435,7 @@ void
 DFT_ground_state::print_info(std::ostream& out__) const
 {
     double evalsum1     = kset_.valence_eval_sum();
-    double evalsum2     = core_eval_sum(ctx_.unit_cell());
+    double evalsum2     = density_.core_eval_sum();
     double s_sum        = kset_.entropy_sum();
     double ekin         = energy_kin(ctx_, kset_, density_, potential_);
     double evxc         = energy_vxc(density_, potential_);
