@@ -22,7 +22,7 @@ namespace sirius {
 class Hubbard_matrix
 {
   protected:
-    Simulation_context& ctx_;
+    Simulation_context const& ctx_;
     /// Local part of Hubbard matrix
     int num_steps_{0};
     double constraint_error_{1.0};
@@ -63,11 +63,14 @@ class Hubbard_matrix
     access(std::string const& what__, std::complex<double>* ptr__, int ld__);
 
     void
-    print_local(int ia__, std::ostream& out__) const;
+    print_local(int idx__, std::ostream& out__) const;
 
     void
     print_nonlocal(int idx__, std::ostream& out__) const;
 
+    void
+    print(std::ostream& out__) const;
+
     void
     zero();
 
