@@ -224,7 +224,10 @@ class Atom_type
      *  oribtal occupancies. */
     std::vector<double> paw_wf_occ_;
 
-    /// Core electron contribution to all electron charge density in PAW method.
+    /// Core electron contribution to all-electron charge density in PAW method.
+    /** In PAW, core charge density is defined for each atom type, because it comes from the
+     *  pseudopotential file and is fixed during the calculation. This is different from LAPW
+     *  core charge density, which is recomputed for each atom symmetry class. */
     std::vector<double> paw_ae_core_charge_density_;
 
     /// True if the pseudo potential includes spin orbit coupling.
@@ -281,13 +284,6 @@ class Atom_type
     inline void
     read_pseudo_paw(nlohmann::json const& parser);
 
-    /// Read atomic parameters from json file.
-    inline void
-    read_input(std::string const& str__);
-
-    inline void
-    read_input(nlohmann::json const& parser);
-
     /// Read atomic parameters directly from UPF v2 files
 #ifdef SIRIUS_USE_PUGIXML
     inline void
@@ -301,31 +297,8 @@ class Atom_type
 #endif
 
     /// Initialize descriptors of the augmented-wave radial functions.
-    inline void
-    init_aw_descriptors()
-    {
-        RTE_ASSERT(this->lmax_apw() >= -1);
-
-        if (this->lmax_apw() >= 0 && aw_default_l_.size() == 0) {
-            RTE_THROW("default AW descriptor is empty");
-        }
-
-        aw_descriptors_.clear();
-        for (int l = 0; l <= this->lmax_apw(); l++) {
-            aw_descriptors_.push_back(aw_default_l_);
-            for (size_t ord = 0; ord < aw_descriptors_[l].size(); ord++) {
-                aw_descriptors_[l][ord].n = l + 1;
-                aw_descriptors_[l][ord].l = l;
-            }
-        }
-
-        for (size_t i = 0; i < aw_specific_l_.size(); i++) {
-            int l = aw_specific_l_[i][0].l;
-            if (l < this->lmax_apw()) {
-                aw_descriptors_[l] = aw_specific_l_[i];
-            }
-        }
-    }
+    void
+    init_aw_descriptors();
 
     /* forbid copy constructor */
     Atom_type(Atom_type const& src) = delete;
@@ -376,6 +349,13 @@ class Atom_type
     /// Move constructor.
     Atom_type(Atom_type&& src) = default;
 
+    /// Read atomic parameters from json file.
+    void
+    read_input(std::string const& str__);
+
+    void
+    read_input(nlohmann::json const& parser);
+
     /// Initialize the atom type.
     /** Once the unit cell is populated with all atom types and atoms, each atom type can be initialized. */
     void
@@ -399,6 +379,28 @@ class Atom_type
     void
     print_info(std::ostream& out__) const;
 
+    /// Serialize atom type information to json
+    nlohmann::json
+    serialize() const;
+
+    /// Add augmented-wave descriptor.
+    void
+    add_aw_descriptor(int n, int l, double enu, int dme, int auto_enu);
+
+    /// Add local orbital descriptor
+    void
+    add_lo_descriptor(int ilo, int n, int l, double enu, int dme, int auto_enu);
+
+    /// Add atomic radial function to the list.
+    void
+    add_ps_atomic_wf(int n__, angular_momentum am__, std::vector<double> f__, double occ__ = 0.0);
+
+    /// Add radial function of the augmentation charge.
+    /** Radial functions of beta projectors must be added already. Their total number will be used to
+        deterimine the storage size for the radial functions of the augmented charge. */
+    inline void
+    add_q_radial_function(int idxrf1__, int idxrf2__, int l__, std::vector<double> qrf__);
+
     /// Set the radial grid of the given type.
     inline void
     set_radial_grid(radial_grid_t grid_type__, int num_points__, double rmin__, double rmax__, double p__)
@@ -442,76 +444,6 @@ class Atom_type
         return atomic_levels_[idx];
     }
 
-    /// Add augmented-wave descriptor.
-    inline void
-    add_aw_descriptor(int n, int l, double enu, int dme, int auto_enu)
-    {
-        if (static_cast<int>(aw_descriptors_.size()) < (l + 1)) {
-            aw_descriptors_.resize(l + 1, radial_solution_descriptor_set());
-        }
-
-        radial_solution_descriptor rsd;
-
-        rsd.n = n;
-        if (n == -1) {
-            /* default principal quantum number value for any l */
-            rsd.n = l + 1;
-            for (int ist = 0; ist < num_atomic_levels(); ist++) {
-                /* take next level after the core */
-                if (atomic_level(ist).core && atomic_level(ist).l == l) {
-                    rsd.n = atomic_level(ist).n + 1;
-                }
-            }
-        }
-
-        rsd.l        = l;
-        rsd.dme      = dme;
-        rsd.enu      = enu;
-        rsd.auto_enu = auto_enu;
-        aw_descriptors_[l].push_back(rsd);
-    }
-
-    /// Add local orbital descriptor
-    inline void
-    add_lo_descriptor(int ilo, int n, int l, double enu, int dme, int auto_enu)
-    {
-        if ((int)lo_descriptors_.size() == ilo) {
-            angular_momentum am(l);
-            lo_descriptors_.push_back(local_orbital_descriptor(am));
-        } else {
-            if (l != lo_descriptors_[ilo].am.l()) {
-                std::stringstream s;
-                s << "wrong angular quantum number" << std::endl
-                  << "atom type id: " << id() << " (" << symbol_ << ")" << std::endl
-                  << "idxlo: " << ilo << std::endl
-                  << "n: " << l << std::endl
-                  << "l: " << n << std::endl
-                  << "expected l: " << lo_descriptors_[ilo].am.l() << std::endl;
-                RTE_THROW(s);
-            }
-        }
-
-        radial_solution_descriptor rsd;
-
-        rsd.n = n;
-        if (n == -1) {
-            /* default value for any l */
-            rsd.n = l + 1;
-            for (int ist = 0; ist < num_atomic_levels(); ist++) {
-                if (atomic_level(ist).core && atomic_level(ist).l == l) {
-                    /* take next level after the core */
-                    rsd.n = atomic_level(ist).n + 1;
-                }
-            }
-        }
-
-        rsd.l        = l;
-        rsd.dme      = dme;
-        rsd.enu      = enu;
-        rsd.auto_enu = auto_enu;
-        lo_descriptors_[ilo].rsd_set.push_back(rsd);
-    }
-
     /// Add the entire local orbital descriptor.
     inline void
     add_lo_descriptor(local_orbital_descriptor const& lod__)
@@ -519,22 +451,6 @@ class Atom_type
         lo_descriptors_.push_back(lod__);
     }
 
-    /// Add atomic radial function to the list.
-    inline void
-    add_ps_atomic_wf(int n__, angular_momentum am__, std::vector<double> f__, double occ__ = 0.0)
-    {
-        Spline<double> rwf(radial_grid_, f__);
-        auto d = std::sqrt(inner(rwf, rwf, 0, radial_grid_.num_points()));
-        if (d < 1e-4) {
-            std::stringstream s;
-            s << "small norm (" << d << ") of radial atomic pseudo wave-function for n=" << n__
-              << " and j=" << am__.j();
-            RTE_THROW(s);
-        }
-
-        ps_atomic_wfs_.emplace_back(n__, am__, occ__, std::move(rwf));
-    }
-
     /// Return a tuple describing a given atomic radial function
     auto const&
     ps_atomic_wf(int idx__) const
@@ -548,7 +464,10 @@ class Atom_type
     add_beta_radial_function(angular_momentum am__, std::vector<double> beta__)
     {
         if (augment_) {
-            RTE_THROW("can't add more beta projectors");
+            std::stringstream s;
+            s << "augmentation charge has already been added" << std::endl
+              << "this fixes the number of beta projectors to " << this->num_beta_radial_functions() << std::endl
+              << "can't add more beta projectors" << std::endl;
         }
         Spline<double> s(radial_grid_, beta__);
         beta_radial_functions_.push_back(std::make_pair(am__, std::move(s)));
@@ -568,40 +487,6 @@ class Atom_type
         return beta_radial_functions_[idxrf__];
     }
 
-    /// Add radial function of the augmentation charge.
-    /** Radial functions of beta projectors must be added already. Their total number will be used to
-        deterimine the storage size for the radial functions of the augmented charge. */
-    inline void
-    add_q_radial_function(int idxrf1__, int idxrf2__, int l__, std::vector<double> qrf__)
-    {
-        /* sanity check */
-        if (l__ > 2 * lmax_beta()) {
-            std::stringstream s;
-            s << "wrong l for Q radial functions of atom type " << label_ << std::endl
-              << "current l: " << l__ << std::endl
-              << "lmax_beta: " << lmax_beta() << std::endl
-              << "maximum allowed l: " << 2 * lmax_beta();
-
-            RTE_THROW(s);
-        }
-
-        if (!augment_) {
-            /* once we add a Q-radial function, we need to augment the charge */
-            augment_ = true;
-            /* number of radial beta-functions */
-            int nbrf              = num_beta_radial_functions();
-            q_radial_functions_l_ = mdarray<Spline<double>, 2>({nbrf * (nbrf + 1) / 2, 2 * lmax_beta() + 1});
-
-            for (int l = 0; l <= 2 * lmax_beta(); l++) {
-                for (int idx = 0; idx < nbrf * (nbrf + 1) / 2; idx++) {
-                    q_radial_functions_l_(idx, l) = Spline<double>(radial_grid_);
-                }
-            }
-        }
-
-        q_radial_functions_l_(packed_index(idxrf1__, idxrf2__), l__) = Spline<double>(radial_grid_, qrf__);
-    }
-
     /// Return true if this atom type has an augementation charge.
     inline bool
     augment() const
